@@ -46,6 +46,7 @@ namespace Mono.Linker.Steps {
 		protected Queue<AttributeProviderPair> _assemblyLevelAttributes;
 		protected Queue<AttributeProviderPair> _lateMarkedAttributes;
 		protected List<TypeDefinition> _typesWithInterfaces;
+		protected List<MethodBody> _unreachableBodies;
 
 		public AnnotationStore Annotations {
 			get { return _context.Annotations; }
@@ -64,6 +65,7 @@ namespace Mono.Linker.Steps {
 			_assemblyLevelAttributes = new Queue<AttributeProviderPair> ();
 			_lateMarkedAttributes = new Queue<AttributeProviderPair> ();
 			_typesWithInterfaces = new List<TypeDefinition> ();
+			_unreachableBodies = new List<MethodBody> ();
 		}
 
 		public virtual void Process (LinkContext context)
@@ -72,6 +74,7 @@ namespace Mono.Linker.Steps {
 
 			Initialize ();
 			Process ();
+			Complete ();
 		}
 
 		void Initialize ()
@@ -93,6 +96,13 @@ namespace Mono.Linker.Steps {
 			}
 		}
 
+		void Complete ()
+		{
+			foreach (var body in _unreachableBodies) {
+				Annotations.SetAction (body.Method, MethodAction.ConvertToThrow);
+			}
+		}
+
 		void InitializeType (TypeDefinition type)
 		{
 			if (type.HasNestedTypes) {
@@ -188,6 +198,7 @@ namespace Mono.Linker.Steps {
 				ProcessQueue ();
 				ProcessVirtualMethods ();
 				ProcessMarkedTypesWithInterfaces ();
+				ProcessPendingBodies ();
 				DoAdditionalProcessing ();
 			}
 
@@ -242,6 +253,17 @@ namespace Mono.Linker.Steps {
 			}
 		}
 
+		void ProcessPendingBodies ()
+		{
+			for (int i = 0; i < _unreachableBodies.Count; i++) {
+				var body = _unreachableBodies [i];
+				if (Annotations.IsInstantiated (body.Method.DeclaringType)) {
+					MarkMethodBody (body);
+					_unreachableBodies.RemoveAt (i--);
+				}
+			}
+		}
+
 		void ProcessVirtualMethod (MethodDefinition method)
 		{
 			var overrides = Annotations.GetOverrides (method);
@@ -449,7 +471,7 @@ namespace Mono.Linker.Steps {
 					continue;
 
 				if (signature == null) {
-					MarkMethod (m);
+					MarkIndirectlyCalledMethod (m);
 					marked = true;
 					continue;
 				}
@@ -469,7 +491,7 @@ namespace Mono.Linker.Steps {
 				if (i < 0)
 					continue;
 
-				MarkMethod (m);
+				MarkIndirectlyCalledMethod (m);
 				marked = true;
 			}
 
@@ -813,6 +835,9 @@ namespace Mono.Linker.Steps {
 
 		bool ProcessLazyAttributes ()
 		{
+			if (Annotations.HasMarkedAnyIndirectlyCalledMethods () && MarkDisablePrivateReflectionAttribute ())
+				return true;
+
 			var startingQueueCount = _assemblyLevelAttributes.Count;
 			if (startingQueueCount == 0)
 				return false;
@@ -1035,6 +1060,7 @@ namespace Mono.Linker.Steps {
 				_typesWithInterfaces.Add (type);
 
 			if (type.HasMethods) {
+				MarkMethodsIf (type.Methods, IsVirtualNeededByTypeDueToPreservedScope);
 				if (ShouldMarkTypeStaticConstructor (type))
 					MarkStaticConstructor (type);
 
@@ -1346,7 +1372,38 @@ namespace Mono.Linker.Steps {
 				MarkType (constraint);
 		}
 
-		bool IsVirtualAndHasPreservedParent (MethodDefinition method)
+		bool IsVirtualNeededByTypeDueToPreservedScope (MethodDefinition method)
+		{
+			if (!method.IsVirtual)
+				return false;
+
+			var base_list = Annotations.GetBaseMethods (method);
+			if (base_list == null)
+				return false;
+
+			foreach (MethodDefinition @base in base_list) {
+				// Just because the type is marked does not mean we need interface methods.
+				// if the type is never instantiated, interfaces will be removed
+				if (@base.DeclaringType.IsInterface)
+					continue;
+				
+				// If the type is marked, we need to keep overrides of abstract members defined in assemblies
+				// that are copied.  However, if the base method is virtual, then we don't need to keep the override
+				// until the type could be instantiated
+				if (!@base.IsAbstract)
+					continue;
+
+				if (IgnoreScope (@base.DeclaringType.Scope))
+					return true;
+
+				if (IsVirtualNeededByTypeDueToPreservedScope (@base))
+					return true;
+			}
+
+			return false;
+		}
+		
+		bool IsVirtualNeededByInstantiatedTypeDueToPreservedScope (MethodDefinition method)
 		{
 			if (!method.IsVirtual)
 				return false;
@@ -1359,7 +1416,7 @@ namespace Mono.Linker.Steps {
 				if (IgnoreScope (@base.DeclaringType.Scope))
 					return true;
 
-				if (IsVirtualAndHasPreservedParent (@base))
+				if (IsVirtualNeededByTypeDueToPreservedScope (@base))
 					return true;
 			}
 
@@ -1683,6 +1740,12 @@ namespace Mono.Linker.Steps {
 				MarkMethod (method);
 		}
 
+		protected void MarkIndirectlyCalledMethod (MethodDefinition method)
+		{
+			MarkMethod (method);
+			Annotations.MarkIndirectlyCalledMethod (method);
+		}
+
 		protected virtual MethodDefinition MarkMethod (MethodReference reference)
 		{
 			reference = GetOriginalMethod (reference);
@@ -1841,7 +1904,7 @@ namespace Mono.Linker.Steps {
 			foreach (var method in type.Methods) {
 				if (method.IsFinalizer ())
 					MarkMethod (method);
-				else if (IsVirtualAndHasPreservedParent (method))
+				else if (IsVirtualNeededByInstantiatedTypeDueToPreservedScope (method))
 					MarkMethod (method);
 			}
 
@@ -1887,23 +1950,47 @@ namespace Mono.Linker.Steps {
 				break;
 
 			case MethodAction.ConvertToThrow:
-				if (_context.MarkedKnownMembers.NotSupportedExceptionCtorString != null)
-					break;
-
-				var nse = BCL.FindPredefinedType ("System", "NotSupportedException", _context);
-				if (nse == null)
-					throw new NotSupportedException ("Missing predefined 'System.NotSupportedException' type");
-
-				MarkType (nse);
-
-				var nseCtor = MarkMethodIf (nse.Methods, KnownMembers.IsNotSupportedExceptionCtorString);
-				if (nseCtor == null)
-					throw new MarkException ($"Could not find constructor on '{nse.FullName}'");
-
-				_context.MarkedKnownMembers.NotSupportedExceptionCtorString = nseCtor;
+				MarkAndCacheConvertToThrowExceptionCtor ();
 				break;
 			}
-		}		
+		}
+
+		protected virtual void MarkAndCacheConvertToThrowExceptionCtor ()
+		{
+			if (_context.MarkedKnownMembers.NotSupportedExceptionCtorString != null)
+				return;
+
+			var nse = BCL.FindPredefinedType ("System", "NotSupportedException", _context);
+			if (nse == null)
+				throw new NotSupportedException ("Missing predefined 'System.NotSupportedException' type");
+
+			MarkType (nse);
+
+			var nseCtor = MarkMethodIf (nse.Methods, KnownMembers.IsNotSupportedExceptionCtorString);
+			if (nseCtor == null)
+				throw new MarkException ($"Could not find constructor on '{nse.FullName}'");
+
+			_context.MarkedKnownMembers.NotSupportedExceptionCtorString = nseCtor;
+		}
+
+		bool MarkDisablePrivateReflectionAttribute ()
+		{
+			if (_context.MarkedKnownMembers.DisablePrivateReflectionAttributeCtor != null)
+				return false;
+
+			var nse = BCL.FindPredefinedType ("System.Runtime.CompilerServices", "DisablePrivateReflectionAttribute", _context);
+			if (nse == null)
+				throw new NotSupportedException ("Missing predefined 'System.Runtime.CompilerServices.DisablePrivateReflectionAttribute' type");
+
+			MarkType (nse);
+
+			var ctor = MarkMethodIf (nse.Methods, MethodDefinitionExtensions.IsDefaultConstructor);
+			if (ctor == null)
+				throw new MarkException ($"Could not find constructor on '{nse.FullName}'");
+
+			_context.MarkedKnownMembers.DisablePrivateReflectionAttributeCtor = ctor;
+			return true;
+		}
 
 		void MarkBaseMethods (MethodDefinition method)
 		{
@@ -2036,6 +2123,12 @@ namespace Mono.Linker.Steps {
 
 		protected virtual void MarkMethodBody (MethodBody body)
 		{
+			if (_context.IsOptimizationEnabled (CodeOptimizations.UnreachableBodies) && IsUnreachableBody (body)) {
+				MarkAndCacheConvertToThrowExceptionCtor ();
+				_unreachableBodies.Add (body);
+				return;
+			}
+
 			foreach (VariableDefinition var in body.Variables)
 				MarkType (var.VariableType);
 
@@ -2053,6 +2146,14 @@ namespace Mono.Linker.Steps {
 			PostMarkMethodBody (body);
 		}
 
+		bool IsUnreachableBody (MethodBody body)
+		{
+			return !body.Method.IsStatic
+				&& !Annotations.IsInstantiated (body.Method.DeclaringType)
+				&& MethodBodyScanner.IsWorthConvertingToThrow (body);
+		}
+		
+
 		partial void PostMarkMethodBody (MethodBody body);
 
 		void MarkInterfacesNeededByBodyStack (MethodBody body)
@@ -2226,7 +2327,7 @@ namespace Mono.Linker.Steps {
 				if ((bindingFlags == BindingFlags.Default || bindingFlags.IsSet(BindingFlags.Public) == method.IsPublic) && method.Name == ".ctor") {
 					Tracer.Push ($"Reflection-{method}");
 					try {
-						MarkMethod (method);
+						MarkIndirectlyCalledMethod (method);
 					} finally {
 						Tracer.Pop ();
 					}
@@ -2244,7 +2345,7 @@ namespace Mono.Linker.Steps {
 					&& method.Name == name) {
 					Tracer.Push ($"Reflection-{method}");
 					try {
-						MarkMethod (method);
+						MarkIndirectlyCalledMethod (method);
 					} finally {
 						Tracer.Pop ();
 					}
@@ -2264,8 +2365,13 @@ namespace Mono.Linker.Steps {
 						// It is not easy to reliably detect in the IL code whether the getter or setter (or both) are used.
 						// Be conservative and mark everything for the property.
 						MarkProperty (property);
-						MarkMethodIfNotNull (property.GetMethod);
-						MarkMethodIfNotNull (property.SetMethod);
+
+						if (property.GetMethod != null)
+							MarkIndirectlyCalledMethod (property.GetMethod);
+
+						if (property.SetMethod != null)
+							MarkIndirectlyCalledMethod (property.SetMethod);
+
 					} finally {
 						Tracer.Pop ();
 					}
