@@ -157,6 +157,7 @@ namespace Mono.Linker {
 				var custom_steps = new List<string> ();
 				var excluded_features = new HashSet<string> (StringComparer.Ordinal);
 				var disabled_optimizations = new HashSet<string> (StringComparer.Ordinal);
+				var enabled_optimizations = new HashSet<string> (StringComparer.Ordinal);
 				bool dumpDependencies = false;
 				bool ignoreDescriptors = false;
 				bool removeCAS = true;
@@ -218,6 +219,10 @@ namespace Mono.Linker {
 							}
 							continue;
 
+						case "--explicit-reflection":
+							context.AddReflectionAnnotations = true;
+							continue;
+
 						case "--custom-step":
 							custom_steps.Add (GetParam ());
 							continue;
@@ -236,6 +241,23 @@ namespace Mono.Linker {
 								disabled_optimizations.Add (opt);
 
 							continue;
+
+						case "--enable-opt":
+							opt = GetParam ().ToLower ();
+							if (!enabled_optimizations.Contains (opt))
+								enabled_optimizations.Add (opt);
+
+							continue;
+
+						case "--new-mvid":
+							if (!bool.Parse (GetParam()))
+								p.RemoveStep (typeof (RegenerateGuidStep));
+							continue;
+
+						case "--deterministic":
+							context.DeterministicOutput = true;
+							p.RemoveStep (typeof (RegenerateGuidStep));
+							continue;
 						}
 
 						switch (token [2]) {
@@ -251,6 +273,12 @@ namespace Mono.Linker {
 						}
 					}
 
+					// Ensure this does not conflict with '-r' below.
+					if (token == "-reference") {
+						context.Resolver.AddReferenceAssembly (GetParam ());
+						continue;
+					}
+
 					switch (token [1]) {
 					case 'd':
 						DirectoryInfo info = new DirectoryInfo (GetParam ());
@@ -328,6 +356,9 @@ namespace Mono.Linker {
 				foreach (string custom_step in custom_steps)
 					AddCustomStep (p, custom_step);
 
+				if (context.AddReflectionAnnotations)
+					p.AddStepAfter (typeof (MarkStep), new ReflectionBlockedStep ());
+
 				p.AddStepAfter (typeof (LoadReferencesStep), new LoadI18nAssemblies (assemblies));
 
 				if (_needAddBypassNGenStep) {
@@ -346,6 +377,7 @@ namespace Mono.Linker {
 					p.AddStepBefore (typeof (MarkStep), new RemoveFeaturesStep () {
 						FeatureCOM = excluded_features.Contains ("com"),
 						FeatureETW = excluded_features.Contains ("etw"),
+						FeatureSRE = excluded_features.Contains ("sre"),
 						FeatureGlobalization = excluded_features.Contains ("globalization")
 					});
 
@@ -363,6 +395,19 @@ namespace Mono.Linker {
 						case "overrideremoval":
 							context.DisabledOptimizations |= CodeOptimizations.OverrideRemoval;
 							break;
+						case "unreachablebodies":
+							context.DisabledOptimizations |= CodeOptimizations.UnreachableBodies;
+							break;
+						}
+					}
+				}
+
+				if (enabled_optimizations.Count > 0) {
+					foreach (var item in enabled_optimizations) {
+						switch (item) {
+						case "unreachablebodies":
+							context.DisabledOptimizations &= ~CodeOptimizations.UnreachableBodies;
+							break;
 						}
 					}
 				}
@@ -456,7 +501,12 @@ namespace Mono.Linker {
 		AssemblyAction ParseAssemblyAction (string s)
 		{
 			var assemblyAction = (AssemblyAction)Enum.Parse(typeof(AssemblyAction), s, true);
-			_needAddBypassNGenStep = ((assemblyAction == AssemblyAction.AddBypassNGen) || (assemblyAction == AssemblyAction.AddBypassNGenUsed));
+			// The AddBypassNGenStep is necessary if any actions (default or per-assembly) are AddBypassNGen(Used).
+			// We enable this step as soon as we see such an action. Even if subsequent parameters change an action we have
+			// already seen, the step will only operate on assemblies with a final action AddBypassNGen(Used).
+			if ((assemblyAction == AssemblyAction.AddBypassNGen) || (assemblyAction == AssemblyAction.AddBypassNGenUsed)) {
+				_needAddBypassNGenStep = true;
+			}
 			return assemblyAction;
 		}
 
@@ -494,15 +544,15 @@ namespace Mono.Linker {
 			Console.WriteLine ("  -r                  Link from a list of assemblies using roots visible outside of the assembly");
 			Console.WriteLine ("  -x                  Link from XML descriptor");
 			Console.WriteLine ("  -d <path>           Specify additional directories to search in for references");
+			Console.WriteLine ("  -reference <file>   Specify additional assemblies to use as references");
 			Console.WriteLine ("  -b                  Update debug symbols for each linked module. Defaults to false");
 			Console.WriteLine ("  -v                  Keep members and types used by debugger. Defaults to false");
 			Console.WriteLine ("  -l <name>,<name>    List of i18n assemblies to copy to the output directory. Defaults to 'all'");
 			Console.WriteLine ("                        Valid names are 'none', 'all', 'cjk', 'mideast', 'other', 'rare', 'west'");
+			Console.WriteLine ("  -out <path>         Specify the output directory. Defaults to 'output'");
 			Console.WriteLine ("  --about             About the {0}", _linker);
 			Console.WriteLine ("  --verbose           Log messages indicating progress and warnings");
 			Console.WriteLine ("  --version           Print the version number of the {0}", _linker);
-			Console.WriteLine ("  --skip-unresolved   Ignore unresolved types, methods, and assemblies. Defaults to false");
-			Console.WriteLine ("  -out <path>         Specify the output directory. Defaults to 'output'");
 
 			Console.WriteLine ();
 			Console.WriteLine ("Actions");
@@ -519,9 +569,11 @@ namespace Mono.Linker {
 			Console.WriteLine ();
 			Console.WriteLine ("Advanced");
 			Console.WriteLine ("  --custom-step <name>      Add a custom step to the pipeline");
+			Console.WriteLine ("  --deterministic           Produce a deterministic output for linked assemblies");
 			Console.WriteLine ("  --disable-opt <name>      Disable one of the default optimizations");
 			Console.WriteLine ("                              beforefieldinit: Unused static fields are removed if there is no static ctor");
 			Console.WriteLine ("                              overrideremoval: Overrides of virtual methods on types that are never instantiated are removed");
+			Console.WriteLine ("                              unreachablebodies: Instance methods that are marked but can never be entered are converted to throws");
 			Console.WriteLine ("  --exclude-feature <name>  Any code which has a feature <name> in linked assemblies will be removed");
 			Console.WriteLine ("                              com: Support for COM Interop");
 			Console.WriteLine ("                              etw: Event Tracing for Windows");
@@ -531,9 +583,11 @@ namespace Mono.Linker {
 			Console.WriteLine ("  --ignore-descriptors      Skips reading embedded descriptors (short -z). Defaults to false");
 			Console.WriteLine ("  --keep-facades            Keep assemblies with type-forwarders (short -t). Defaults to false");
 			Console.WriteLine ("  --new-mvid                Generate a new guid for each linked assembly (short -g). Defaults to true");
+			Console.WriteLine ("  --skip-unresolved         Ignore unresolved types, methods, and assemblies. Defaults to false");			
 			Console.WriteLine ("  --strip-resources         Remove XML descriptor resources for linked assemblies. Defaults to true");
 			Console.WriteLine ("  --strip-security          Remove metadata and code related to Code Access Security. Defaults to true");
 			Console.WriteLine ("  --used-attrs-only         Any attribute is removed if the attribute type is not used. Defaults to false");
+			Console.WriteLine ("  --explicit-reflection     Adds to members never used through reflection DisablePrivateReflection attribute. Defaults to false");
 
 			Console.WriteLine ();
 			Console.WriteLine ("Analyzer");
