@@ -794,6 +794,9 @@ has_cattr_get_table (MonoImage *m, guint32 val)
 	case MONO_CUSTOM_ATTR_GENERICPAR:
 		table = "GenericParam";
 		break;
+	case MONO_CUSTOM_ATTR_GENERICPARAMCONSTRAINT:
+		table = "GenericParamConstraint";
+		break;
 	default:
 		table = "Unknown";
 		break;
