@@ -21,6 +21,7 @@
 #include <mono/metadata/object-internals.h>
 #include <mono/metadata/metadata-internals.h>
 #include <mono/metadata/appdomain.h>
+#include <mono/metadata/domain-internals.h>
 #include <mono/metadata/mono-debug.h>
 #include <mono/utils/mono-error-internals.h>
 #include <mono/utils/mono-logger-internals.h>
@@ -102,14 +103,14 @@ mono_exception_new_by_name_domain (MonoDomain *domain, MonoImage *image,
 	goto_if_nok (error, return_null);
 
 	if (domain != caller_domain)
-		mono_domain_set_internal (domain);
+		mono_domain_set_internal_with_options (domain, TRUE);
 
 	mono_runtime_object_init_handle (o, error);
 	mono_error_assert_ok (error);
 
 	// Restore domain in success and error path.
 	if (domain != caller_domain)
-		mono_domain_set_internal (caller_domain);
+		mono_domain_set_internal_with_options (caller_domain, TRUE);
 
 	goto_if_ok (error, exit);
 return_null:
@@ -678,11 +679,22 @@ mono_get_exception_argument (const char *arg, const char *msg)
 	return mono_get_exception_argument_internal ("ArgumentException", arg, msg);
 }
 
+#ifndef ENABLE_NETCORE
 TYPED_HANDLE_DECL (MonoArgumentException);
+#endif
 
 static MonoExceptionHandle
 mono_exception_new_argument_internal (const char *type, const char *arg, const char *msg, MonoError *error)
 {
+#ifdef ENABLE_NETCORE
+	MonoStringHandle arg_str = arg ? mono_string_new_handle (mono_domain_get (), arg, error) : NULL_HANDLE_STRING;
+	MonoStringHandle msg_str = msg ? mono_string_new_handle (mono_domain_get (), msg, error) : NULL_HANDLE_STRING;
+
+	if (!strcmp (type, "ArgumentException"))
+		return mono_exception_from_name_two_strings_checked (mono_get_corlib (), "System", type, msg_str, arg_str, error);
+	else
+		return mono_exception_from_name_two_strings_checked (mono_get_corlib (), "System", type, arg_str, msg_str, error);
+#else
 	MonoExceptionHandle ex = mono_exception_new_by_name_msg (mono_get_corlib (), "System", type, msg, error);
 
 	if (arg && !MONO_HANDLE_IS_NULL (ex)) {
@@ -690,8 +702,8 @@ mono_exception_new_argument_internal (const char *type, const char *arg, const c
 		MonoStringHandle arg_str = mono_string_new_handle (MONO_HANDLE_DOMAIN (ex), arg, error);
 		MONO_HANDLE_SET (argex, param_name, arg_str);
 	}
-
 	return ex;
+#endif
 }
 
 MonoExceptionHandle
@@ -706,6 +718,12 @@ mono_exception_new_argument_null (const char *arg, MonoError *error)
 	return mono_exception_new_argument_internal ("ArgumentNullException", arg, NULL, error);
 }
 
+MonoExceptionHandle
+mono_exception_new_argument_out_of_range(const char *arg, const char *msg, MonoError *error)
+{
+	return mono_exception_new_argument_internal ("ArgumentOutOfRangeException", arg, msg, error);
+}
+
 MonoExceptionHandle
 mono_exception_new_serialization (const char *msg, MonoError *error)
 {
@@ -1480,11 +1498,13 @@ mono_error_set_simple_file_not_found (MonoError *error, const char *file_name, g
 }
 
 void
-mono_error_set_argument_out_of_range (MonoError *error, const char *name)
+mono_error_set_argument_out_of_range (MonoError *error, const char *param_name, const char *msg_format, ...)
 {
-	ERROR_DECL (error_creating_exception);
-	mono_error_set_exception_handle (error, mono_new_exception_argument_out_of_range (name, error_creating_exception));
-	mono_error_cleanup (error_creating_exception);
+	char *str;
+	SET_ERROR_MSG (str, msg_format);
+	mono_error_set_specific (error, MONO_ERROR_ARGUMENT_OUT_OF_RANGE, str);
+	if (param_name)
+		mono_error_set_first_argument (error, param_name);
 }
 
 MonoExceptionHandle
