@@ -257,7 +257,7 @@ mono_gc_run_finalize (void *obj, void *data)
 	/* g_print ("Finalize run on %p %s.%s\n", o, mono_object_class (o)->name_space, mono_object_class (o)->name); */
 
 	/* Use _internal here, since this thread can enter a doomed appdomain */
-	mono_domain_set_internal (mono_object_domain (o));
+	mono_domain_set_internal_with_options (mono_object_domain (o), TRUE);
 
 	/* delegates that have a native function pointer allocated are
 	 * registered for finalization, but they don't have a Finalize
@@ -267,7 +267,7 @@ mono_gc_run_finalize (void *obj, void *data)
 		MonoDelegate* del = (MonoDelegate*)o;
 		if (del->delegate_trampoline)
 			mono_delegate_free_ftnptr ((MonoDelegate*)o);
-		mono_domain_set_internal (caller_domain);
+		mono_domain_set_internal_with_options (caller_domain, TRUE);
 		return;
 	}
 
@@ -280,7 +280,7 @@ mono_gc_run_finalize (void *obj, void *data)
 	 * of finalizer on object with CCW.
 	 */
 	if (mono_marshal_free_ccw (o) && !finalizer) {
-		mono_domain_set_internal (caller_domain);
+		mono_domain_set_internal_with_options (caller_domain, TRUE);
 		return;
 	}
 
@@ -339,7 +339,7 @@ unhandled_error:
 	if (exc)
 		mono_thread_internal_unhandled_exception (exc);
 
-	mono_domain_set_internal (caller_domain);
+	mono_domain_set_internal_with_options (caller_domain, TRUE);
 }
 
 /*
@@ -615,9 +615,7 @@ ves_icall_System_GC_WaitForPendingFinalizers (void)
 	ResetEvent (pending_done_event);
 	mono_gc_finalize_notify ();
 	/* g_print ("Waiting for pending finalizers....\n"); */
-	MONO_ENTER_GC_SAFE;
-	mono_win32_wait_for_single_object_ex (pending_done_event, INFINITE, TRUE);
-	MONO_EXIT_GC_SAFE;
+	mono_coop_win32_wait_for_single_object_ex (pending_done_event, INFINITE, TRUE);
 	/* g_print ("Done pending....\n"); */
 #else
 	gboolean alerted = FALSE;
@@ -646,6 +644,53 @@ ves_icall_System_GC_get_ephemeron_tombstone (MonoError *error)
 	return MONO_HANDLE_NEW (MonoObject, mono_domain_get ()->ephemeron_tombstone);
 }
 
+#if ENABLE_NETCORE
+
+gpointer
+ves_icall_System_GCHandle_InternalAlloc (MonoObjectHandle obj, gint32 type, MonoError *error)
+{
+	guint32 handle = 0;
+
+	switch (type) {
+	case HANDLE_WEAK:
+		handle = mono_gchandle_new_weakref_from_handle (obj);
+		break;
+	case HANDLE_WEAK_TRACK:
+		handle = mono_gchandle_new_weakref_from_handle_track_resurrection (obj);
+		break;
+	case HANDLE_NORMAL:
+		handle = mono_gchandle_from_handle (obj, FALSE);
+		break;
+	case HANDLE_PINNED:
+		handle = mono_gchandle_from_handle (obj, TRUE);
+		break;
+	default:
+		g_assert_not_reached ();
+	}
+	/* The lowest bit is used to mark pinned handles by netcore's GCHandle class */
+	return GUINT_TO_POINTER (handle << 1);
+}
+
+void
+ves_icall_System_GCHandle_InternalFree (gpointer handle, MonoError *error)
+{
+	mono_gchandle_free_internal (GPOINTER_TO_UINT (handle) >> 1);
+}
+
+MonoObjectHandle
+ves_icall_System_GCHandle_InternalGet (gpointer handle, MonoError *error)
+{
+	return mono_gchandle_get_target_handle (GPOINTER_TO_UINT (handle) >> 1);
+}
+
+void
+ves_icall_System_GCHandle_InternalSet (gpointer handle, MonoObjectHandle obj, MonoError *error)
+{
+	mono_gchandle_set_target_handle (GPOINTER_TO_UINT (handle) >> 1, obj);
+}
+
+#else
+
 MonoObjectHandle
 ves_icall_System_GCHandle_GetTarget (guint32 handle, MonoError *error)
 {
@@ -722,6 +767,8 @@ ves_icall_System_GCHandle_CheckCurrentDomain (guint32 gchandle)
 	return mono_gchandle_is_in_domain (gchandle, mono_domain_get ());
 }
 
+#endif
+
 static MonoCoopSem finalizer_sem;
 static volatile gboolean finished;
 
