@@ -20,7 +20,7 @@ typedef struct {
 } MonoIcallTableCallbacks;
 
 void
-mono_install_icall_table_callbacks (MonoIcallTableCallbacks *cb);
+mono_install_icall_table_callbacks (const MonoIcallTableCallbacks *cb);
 
 MONO_API void
 mono_icall_table_init (void);
@@ -29,12 +29,11 @@ mono_icall_table_init (void);
 // Marshaling a "ptr" does nothing -- just pass it on unchanged.
 // Marshaling a "ref" also does nothing at this layer, but
 // creates a handle in  marshal-ilgen.c.
+// "ref" means "can be an interior pointer".
+// "ptr" means "to a local".
+// It is generally difficult to know, and "ref" is safer.
+// Presently it does not matter.
 typedef gint32  *gint32_ptr;
-typedef gint64  *gint64_ptr;
-typedef guint   *guint_ptr;
-typedef guint32 *guint32_ptr;
-typedef guint64 *guint64_ptr;
-typedef gsize *gsize_ptr;
 typedef guchar *guchar_ptr;
 typedef const guchar *const_guchar_ptr;
 typedef gpointer *gpointer_ptr;
@@ -54,7 +53,6 @@ typedef MonoClassField *MonoClassField_ptr;
 typedef MonoEvent *MonoEvent_ptr;
 typedef MonoImage *MonoImage_ptr;
 typedef MonoMethod *MonoMethod_ptr;
-typedef MonoNativeOverlapped *MonoNativeOverlapped_ptr;
 typedef MonoProperty *MonoProperty_ptr;
 typedef MonoPropertyInfo *MonoPropertyInfo_ref;
 typedef MonoType *MonoType_ptr;
@@ -81,7 +79,6 @@ typedef MonoEvent *MonoEvent_ref;
 typedef MonoEventInfo *MonoEventInfo_ref;
 typedef MonoMethod *MonoMethod_ref;
 typedef MonoMethodInfo *MonoMethodInfo_ref;
-typedef MonoNativeOverlapped *MonoNativeOverlapped_ref;
 typedef MonoResolveTokenError *MonoResolveTokenError_ref;
 typedef MonoType *MonoType_ref;
 typedef MonoTypedRef *MonoTypedRef_ref;
@@ -156,7 +153,6 @@ typedef MonoReflectionModuleHandle MonoReflectionModuleOutHandle;
 #define MONO_HANDLE_TYPE_WRAP_MonoEventInfo_ref		ICALL_HANDLES_WRAP_VALUETYPE_REF
 #define MONO_HANDLE_TYPE_WRAP_MonoMethod_ref 		ICALL_HANDLES_WRAP_VALUETYPE_REF
 #define MONO_HANDLE_TYPE_WRAP_MonoMethodInfo_ref	ICALL_HANDLES_WRAP_VALUETYPE_REF
-#define MONO_HANDLE_TYPE_WRAP_MonoNativeOverlapped_ref	ICALL_HANDLES_WRAP_VALUETYPE_REF
 #define MONO_HANDLE_TYPE_WRAP_MonoPropertyInfo_ref	ICALL_HANDLES_WRAP_VALUETYPE_REF
 #define MONO_HANDLE_TYPE_WRAP_MonoType_ref  		ICALL_HANDLES_WRAP_VALUETYPE_REF
 #define MONO_HANDLE_TYPE_WRAP_MonoTypedRef_ref 		ICALL_HANDLES_WRAP_VALUETYPE_REF
@@ -167,6 +163,7 @@ typedef MonoReflectionModuleHandle MonoReflectionModuleOutHandle;
 #define MONO_HANDLE_TYPE_WRAP_guint32_ref   		ICALL_HANDLES_WRAP_VALUETYPE_REF
 #define MONO_HANDLE_TYPE_WRAP_guint64_ref   		ICALL_HANDLES_WRAP_VALUETYPE_REF
 #define MONO_HANDLE_TYPE_WRAP_int_ref  			ICALL_HANDLES_WRAP_VALUETYPE_REF
+#define MONO_HANDLE_TYPE_WRAP_gint32_ref  			ICALL_HANDLES_WRAP_VALUETYPE_REF
 #define MONO_HANDLE_TYPE_WRAP_int_ptr_ref  		ICALL_HANDLES_WRAP_VALUETYPE_REF
 #define MONO_HANDLE_TYPE_WRAP_char_ptr_ref		ICALL_HANDLES_WRAP_VALUETYPE_REF
 #define MONO_HANDLE_TYPE_WRAP_guint8_ptr_ref		ICALL_HANDLES_WRAP_VALUETYPE_REF
@@ -178,13 +175,10 @@ typedef MonoReflectionModuleHandle MonoReflectionModuleOutHandle;
 #define MONO_HANDLE_TYPE_WRAP_MonoEvent_ptr		ICALL_HANDLES_WRAP_NONE
 #define MONO_HANDLE_TYPE_WRAP_MonoGenericParamInfo_ptr	ICALL_HANDLES_WRAP_NONE
 #define MONO_HANDLE_TYPE_WRAP_MonoMethod_ptr 		ICALL_HANDLES_WRAP_NONE
-#define MONO_HANDLE_TYPE_WRAP_MonoNativeOverlapped_ptr	ICALL_HANDLES_WRAP_NONE
 #define MONO_HANDLE_TYPE_WRAP_MonoType_ptr  		ICALL_HANDLES_WRAP_NONE
 #define MONO_HANDLE_TYPE_WRAP_MonoTypedRef_ptr 		ICALL_HANDLES_WRAP_NONE
 #define MONO_HANDLE_TYPE_WRAP_MonoStackCrawlMark_ptr  	ICALL_HANDLES_WRAP_NONE
 #define MONO_HANDLE_TYPE_WRAP_gint32_ptr   		ICALL_HANDLES_WRAP_NONE
-#define MONO_HANDLE_TYPE_WRAP_guint32_ptr   		ICALL_HANDLES_WRAP_NONE
-#define MONO_HANDLE_TYPE_WRAP_guint64_ptr   		ICALL_HANDLES_WRAP_NONE
 #define MONO_HANDLE_TYPE_WRAP_gpointer_ptr		ICALL_HANDLES_WRAP_NONE
 
 // Please keep this sorted (grep ICALL_HANDLES_WRAP_OBJ$ | sort)
@@ -470,7 +464,7 @@ func ( MONO_HANDLE_FOREACH_ARG_RAWPOINTER_ ## n argtypes)
 // Or put the handles directly in the coop frame, or pointers to them.
 // i.e. one TLS access at function start and end.
 //
-// This is for functions passed to mono_register_jit_icall, etc.
+// This is for functions passed to mono_register_jit_icall_info, etc.
 
 #define MONO_HANDLE_REGISTER_ICALL_IMPLEMENT(func, rettype, n, argtypes)	\
 										\
