@@ -15,6 +15,9 @@
  */
 
 #include <config.h>
+
+#if !ENABLE_NETCORE
+
 #include <glib.h>
 #include <string.h>
 
@@ -284,59 +287,39 @@ ves_icall_System_Globalization_CultureData_fill_culture_data (MonoCultureData *t
 	this_obj->CalendarWeekRule = dfe->calendar_week_rule;
 }
 
-void
-ves_icall_System_Globalization_CultureData_fill_number_data (MonoNumberFormatInfo* number, gint32 number_index)
+gconstpointer
+ves_icall_System_Globalization_CultureData_fill_number_data (gint32 number_index,
+	NumberFormatEntryManaged *managed)
 {
-	ERROR_DECL (error);
-	MonoDomain *domain;
-	const NumberFormatEntry *nfe;
-
-	g_assert (number_index >= 0);
-
-	nfe = &number_format_entries [number_index];
-
-	domain = mono_domain_get ();
-
-	number->currencyDecimalDigits = nfe->currency_decimal_digits;
-
-#define SET_STR(obj,field,domain,expr,err) do {				\
-		MonoString *_tmp_str = mono_string_new_checked ((domain), (expr), (err)); \
-		if (mono_error_set_pending_exception ((err)))		\
-			return;						\
-		MONO_OBJECT_SETREF_INTERNAL ((obj), field, _tmp_str);		\
-	} while (0)
-
-	SET_STR (number, currencyDecimalSeparator, domain, idx2string (nfe->currency_decimal_separator), error);
-	SET_STR (number, currencyGroupSeparator, domain, idx2string (nfe->currency_group_separator), error);
-
-	MonoArray *currency_sizes_arr = create_group_sizes_array (nfe->currency_group_sizes,
-								  GROUP_SIZE, error);
-	if (mono_error_set_pending_exception (error))
-		return;
-	MONO_OBJECT_SETREF_INTERNAL (number, currencyGroupSizes, currency_sizes_arr);
-	number->currencyNegativePattern = nfe->currency_negative_pattern;
-	number->currencyPositivePattern = nfe->currency_positive_pattern;
-
-	SET_STR (number, currencySymbol, domain, idx2string (nfe->currency_symbol), error);
-	SET_STR (number, naNSymbol, domain, idx2string (nfe->nan_symbol), error);
-	SET_STR (number, negativeInfinitySymbol, domain, idx2string (nfe->negative_infinity_symbol), error);
-	SET_STR (number, negativeSign, domain, idx2string (nfe->negative_sign), error);
-	number->numberDecimalDigits = nfe->number_decimal_digits;
-	SET_STR (number, numberDecimalSeparator, domain, idx2string (nfe->number_decimal_separator), error);
-	SET_STR (number, numberGroupSeparator, domain, idx2string (nfe->number_group_separator), error);
-	MonoArray *number_sizes_arr = create_group_sizes_array (nfe->number_group_sizes,
-								GROUP_SIZE, error);
-	if (mono_error_set_pending_exception (error))
-		return;
-	MONO_OBJECT_SETREF_INTERNAL (number, numberGroupSizes, number_sizes_arr);
-	number->numberNegativePattern = nfe->number_negative_pattern;
-	number->percentNegativePattern = nfe->percent_negative_pattern;
-	number->percentPositivePattern = nfe->percent_positive_pattern;
-	SET_STR (number, percentSymbol, domain, idx2string (nfe->percent_symbol), error);
-	SET_STR (number, perMilleSymbol, domain, idx2string (nfe->per_mille_symbol), error);
-	SET_STR (number, positiveInfinitySymbol, domain, idx2string (nfe->positive_infinity_symbol), error);
-	SET_STR (number, positiveSign, domain, idx2string (nfe->positive_sign), error);
-#undef SET_STR
+	g_assertf (number_index >= 0 && number_index < G_N_ELEMENTS (number_format_entries), "%d", number_index);
+	NumberFormatEntry const * const native = &number_format_entries [number_index];
+	// We could return the pointer directly, but I'm leary of the 7x byte layout.
+	// Does C# match C exactly?
+	// If the data is regenerated, suggest a byte of padding there.
+	managed->currency_decimal_digits = native->currency_decimal_digits;
+	managed->currency_decimal_separator = native->currency_decimal_separator;
+	managed->currency_group_separator = native->currency_group_separator;
+	managed->currency_group_sizes0 = native->currency_group_sizes [0];
+	managed->currency_group_sizes1 = native->currency_group_sizes [1];
+	managed->currency_negative_pattern = native->currency_negative_pattern;
+	managed->currency_positive_pattern = native->currency_positive_pattern;
+	managed->currency_symbol = native->currency_symbol;
+	managed->nan_symbol = native->nan_symbol;
+	managed->negative_infinity_symbol = native->negative_infinity_symbol;
+	managed->negative_sign = native->negative_sign;
+	managed->number_decimal_digits = native->number_decimal_digits;
+	managed->number_decimal_separator = native->number_decimal_separator;
+	managed->number_group_separator = native->number_group_separator;
+	managed->number_group_sizes0 = native->number_group_sizes [0];
+	managed->number_group_sizes1  = native->number_group_sizes [1];
+	managed->number_negative_pattern = native->number_negative_pattern;
+	managed->per_mille_symbol = native->per_mille_symbol;
+	managed->percent_negative_pattern = native->percent_negative_pattern;
+	managed->percent_positive_pattern = native->percent_positive_pattern;
+	managed->percent_symbol = native->percent_symbol;
+	managed->positive_infinity_symbol = native->positive_infinity_symbol;
+	managed->positive_sign = native->positive_sign;
+	return locale_strings;
 }
 
 static MonoBoolean
@@ -807,9 +790,9 @@ ves_icall_System_Globalization_CompareInfo_internal_compare (const gunichar2 *us
 	return(string_invariant_compare_char(ustr1[pos], ustr2[pos], options));
 }
 
-int
+gint32
 ves_icall_System_Globalization_CompareInfo_internal_index (const gunichar2 *src, gint32 sindex,
-	gint32 count, const gunichar2 *cmpstr, int lencmpstr, MonoBoolean first)
+	gint32 count, const gunichar2 *cmpstr, gint32 lencmpstr, MonoBoolean first)
 {
 	gint32 pos,i;
 	
@@ -857,3 +840,5 @@ void ves_icall_System_Text_Normalization_load_normalization_resource (guint8 **a
 	*argCombiningClass = (guint8*)combiningClass;
 #endif
 }
+
+#endif /* !ENABLE_NETCORE */
