@@ -1194,7 +1194,7 @@ ves_icall_System_Threading_Monitor_Monitor_try_enter_with_atomic_var (MonoObject
 void
 mono_monitor_enter_v4 (MonoObject *obj, char *lock_taken)
 {
-	g_assert (sizeof (MonoBoolean) == 1); // FIXME static_assert
+	g_static_assert (sizeof (MonoBoolean) == 1);
 	mono_monitor_enter_v4_internal  (obj, (MonoBoolean*)lock_taken);
 }
 
@@ -1382,13 +1382,7 @@ ves_icall_System_Threading_Monitor_Monitor_wait (MonoObject *obj, guint32 ms)
 	 * is private to this thread.  Therefore even if the event was
 	 * signalled before we wait, we still succeed.
 	 */
-#ifdef HOST_WIN32
-	MONO_ENTER_GC_SAFE;
-	ret = mono_w32handle_convert_wait_ret (mono_win32_wait_for_single_object_ex (event, ms, TRUE), 1);
-	MONO_EXIT_GC_SAFE;
-#else
 	ret = mono_w32handle_wait_one (event, ms, TRUE);
-#endif /* HOST_WIN32 */
 
 	/* Reset the thread state fairly early, so we don't have to worry
 	 * about the monitor error checking
@@ -1411,13 +1405,7 @@ ves_icall_System_Threading_Monitor_Monitor_wait (MonoObject *obj, guint32 ms)
 		/* Poll the event again, just in case it was signalled
 		 * while we were trying to regain the monitor lock
 		 */
-#ifdef HOST_WIN32
-		MONO_ENTER_GC_SAFE;
-		ret = mono_w32handle_convert_wait_ret (mono_win32_wait_for_single_object_ex (event, 0, FALSE), 1);
-		MONO_EXIT_GC_SAFE;
-#else
 		ret = mono_w32handle_wait_one (event, 0, FALSE);
-#endif /* HOST_WIN32 */
 	}
 
 	/* Pulse will have popped our event from the queue if it signalled
