@@ -16,6 +16,7 @@
 #include "mono/metadata/assembly.h"
 #include "mono/metadata/loader.h"
 #include "mono/metadata/mono-config.h"
+#include "mono/metadata/mono-config-internals.h"
 #include "mono/metadata/metadata-internals.h"
 #include "mono/metadata/object-internals.h"
 #include "mono/utils/mono-logger-internals.h"
@@ -99,8 +100,6 @@
 #endif
 #endif
 
-static void mono_config_for_assembly_internal (MonoImage *assembly);
-
 /**
  * mono_config_get_os:
  *
@@ -608,17 +607,6 @@ mono_config_string_for_assembly_file (const char *filename)
 	return NULL;
 }
 
-/**
- * mono_config_for_assembly:
- */
-void 
-mono_config_for_assembly (MonoImage *assembly)
-{
-	MONO_ENTER_GC_UNSAFE;
-	mono_config_for_assembly_internal (assembly);
-	MONO_EXIT_GC_UNSAFE;
-}
-
 void
 mono_config_for_assembly_internal (MonoImage *assembly)
 {
@@ -642,14 +630,14 @@ mono_config_for_assembly_internal (MonoImage *assembly)
 	g_free (cfg_name);
 
 	cfg_name = g_strdup_printf ("%s.config", mono_image_get_name (assembly));
-	const char *mono_cfg_dir = mono_get_config_dir ();
-	if (!mono_cfg_dir) {
+	const char *cfg_dir = mono_get_config_dir ();
+	if (!cfg_dir) {
 		g_free (cfg_name);
 		return;
 	}
 
 	for (i = 0; (aname = get_assembly_filename (assembly, i)) != NULL; ++i) {
-		cfg = g_build_filename (mono_cfg_dir, "mono", "assemblies", aname, cfg_name, NULL);
+		cfg = g_build_filename (cfg_dir, "mono", "assemblies", aname, cfg_name, NULL);
 		got_it += mono_config_parse_file_with_context (&state, cfg);
 		g_free (cfg);
 
@@ -692,9 +680,9 @@ mono_config_parse (const char *filename) {
 		return;
 	}
 
-	const char *mono_cfg_dir = mono_get_config_dir ();
-	if (mono_cfg_dir) {
-		mono_cfg = g_build_filename (mono_cfg_dir, "mono", "config", NULL);
+	const char *cfg_dir = mono_get_config_dir ();
+	if (cfg_dir) {
+		mono_cfg = g_build_filename (cfg_dir, "mono", "config", NULL);
 		mono_config_parse_file (mono_cfg);
 		g_free (mono_cfg);
 	}
