@@ -28,6 +28,7 @@
 #include <glib.h>
 
 #include <mono/metadata/class-internals.h>
+#include <mono/metadata/domain-internals.h>
 #include <mono/metadata/exception.h>
 #include <mono/metadata/gc-internals.h>
 #include <mono/metadata/object.h>
@@ -184,20 +185,17 @@ mono_threadpool_enqueue_work_item (MonoDomain *domain, MonoObject *work_item, Mo
 	current_domain = mono_domain_get ();
 	if (current_domain == domain) {
 		mono_runtime_invoke_checked (unsafe_queue_custom_work_item_method, NULL, args, error);
-		return_val_if_nok (error, FALSE);
 	} else {
 		mono_thread_push_appdomain_ref (domain);
-		if (mono_domain_set (domain, FALSE)) {
+		if (mono_domain_set_fast (domain, FALSE)) {
 			mono_runtime_invoke_checked (unsafe_queue_custom_work_item_method, NULL, args, error);
-			if (!is_ok (error)) {
-				mono_thread_pop_appdomain_ref ();
-				return FALSE;
-			}
-			mono_domain_set (current_domain, TRUE);
+			mono_domain_set_fast (current_domain, TRUE);
+		} else {
+			// mono_domain_set_fast failing still leads to success.
 		}
 		mono_thread_pop_appdomain_ref ();
 	}
-	return TRUE;
+	return is_ok (error);
 }
 
 /* LOCKING: domains_lock must be held. */
@@ -283,7 +281,7 @@ try_invoke_perform_wait_callback (MonoObject** exc, MonoError *error)
 {
 	HANDLE_FUNCTION_ENTER ();
 	error_init (error);
-	MonoObject *res = mono_runtime_try_invoke (mono_defaults.threadpool_perform_wait_callback_method, NULL, NULL, exc, error);
+	MonoObject * const res = mono_runtime_try_invoke (mono_defaults.threadpool_perform_wait_callback_method, NULL, NULL, exc, error);
 	HANDLE_FUNCTION_RETURN_VAL (res);
 }
 
@@ -364,7 +362,7 @@ worker_callback (void)
 			ThreadState_Background);
 
 		mono_thread_push_appdomain_ref (tpdomain->domain);
-		if (mono_domain_set (tpdomain->domain, FALSE)) {
+		if (mono_domain_set_fast (tpdomain->domain, FALSE)) {
 			MonoObject *exc = NULL, *res;
 
 			res = try_invoke_perform_wait_callback (&exc, error);
@@ -378,7 +376,7 @@ worker_callback (void)
 				retire = TRUE;
 			}
 
-			mono_domain_set (mono_get_root_domain (), TRUE);
+			mono_domain_set_fast (mono_get_root_domain (), TRUE);
 		}
 		mono_thread_pop_appdomain_ref ();
 
@@ -501,13 +499,7 @@ mono_threadpool_end_invoke (MonoAsyncResult *ares, MonoArray **out_args, MonoObj
 			MONO_OBJECT_SETREF_INTERNAL (ares, handle, (MonoObject*) wait_handle);
 		}
 		mono_monitor_exit_internal ((MonoObject*) ares);
-#ifdef HOST_WIN32
-		MONO_ENTER_GC_SAFE;
-		mono_win32_wait_for_single_object_ex (wait_event, INFINITE, TRUE);
-		MONO_EXIT_GC_SAFE;
-#else
 		mono_w32handle_wait_one (wait_event, MONO_INFINITE_WAIT, TRUE);
-#endif
 	}
 
 	ac = (MonoAsyncCall*) ares->object_data;
@@ -807,24 +799,3 @@ ves_icall_System_Threading_ThreadPool_RequestWorkerThread (MonoError *error)
 	mono_refcount_dec (&threadpool);
 	return TRUE;
 }
-
-MonoBoolean G_GNUC_UNUSED
-ves_icall_System_Threading_ThreadPool_PostQueuedCompletionStatus (MonoNativeOverlapped *native_overlapped, MonoError *error)
-{
-	/* This copy the behavior of the current Mono implementation */
-	mono_error_set_not_implemented (error, "");
-	return FALSE;
-}
-
-MonoBoolean G_GNUC_UNUSED
-ves_icall_System_Threading_ThreadPool_BindIOCompletionCallbackNative (gpointer file_handle, MonoError *error)
-{
-	/* This copy the behavior of the current Mono implementation */
-	return TRUE;
-}
-
-MonoBoolean G_GNUC_UNUSED
-ves_icall_System_Threading_ThreadPool_IsThreadPoolHosted (MonoError *error)
-{
-	return FALSE;
-}
