@@ -215,6 +215,9 @@ gint32 ves_icall_System_Threading_Interlocked_Decrement_Int(gint32 *location);
 ICALL_EXPORT
 gint64 ves_icall_System_Threading_Interlocked_Decrement_Long(gint64 * location);
 
+ICALL_EXPORT
+void ves_icall_System_Threading_Interlocked_MemoryBarrierProcessWide (void);
+
 ICALL_EXPORT
 gint8 ves_icall_System_Threading_Thread_VolatileRead1 (void *ptr);
 
@@ -275,6 +278,9 @@ gint32 ves_icall_System_Threading_Volatile_Read4 (void *ptr);
 ICALL_EXPORT
 gint64 ves_icall_System_Threading_Volatile_Read8 (void *ptr);
 
+ICALL_EXPORT
+guint64 ves_icall_System_Threading_Volatile_ReadU8 (void *ptr);
+
 ICALL_EXPORT
 void * ves_icall_System_Threading_Volatile_ReadIntPtr (void *ptr);
 
@@ -299,6 +305,9 @@ void ves_icall_System_Threading_Volatile_Write4 (void *ptr, gint32);
 ICALL_EXPORT
 void ves_icall_System_Threading_Volatile_Write8 (void *ptr, gint64);
 
+ICALL_EXPORT
+void ves_icall_System_Threading_Volatile_WriteU8 (void *ptr, guint64);
+
 ICALL_EXPORT
 void ves_icall_System_Threading_Volatile_WriteIntPtr (void *ptr, void *);
 
