@@ -1397,13 +1397,7 @@ disasm_ins (MonoMethod *method, const guchar *ip, const guint8 **endip)
 
 		switch (ip [1]) {
 		case CEE_MONO_ICALL: {
-			MonoJitICallInfo *info;
-
-			token = read32 (ip + 2);
-			data = mono_method_get_wrapper_data (method, token);
-			info = mono_find_jit_icall_by_addr (data);
-			g_assert (info);
-
+			MonoJitICallInfo const * const info = mono_find_jit_icall_info ((MonoJitICallId)read32 (ip + 2));
 			dis = g_strdup_printf ("IL_%04x: mono_icall <%s>", (int)(ip - header->code), info->name);
 			ip += 6;
 			break;
