@@ -769,33 +769,37 @@ mini_llvmonly_resolve_iface_call_gsharedvt (MonoObject *this_obj, int imt_slot,
 }
 
 static void
-init_llvmonly_method (MonoAotModule *amodule, guint32 method_index, MonoClass *init_class, MonoGenericContext *context,
-					  gboolean lookup_context)
+init_llvmonly_method (MonoAotModule *amodule, guint32 method_index, MonoClass *init_class)
 {
 	ERROR_DECL (error);
 	gboolean res;
 
-	res = mono_aot_init_llvmonly_method (amodule, method_index, init_class, context, lookup_context, error);
+	res = mono_aot_init_llvmonly_method (amodule, method_index, init_class, error);
 	if (!res || !is_ok (error)) {
 		MonoException *ex = mono_error_convert_to_exception (error);
-		/* Its okay to raise in llvmonly mode */
-		if (ex)
-			mono_llvm_throw_exception ((MonoObject*)ex);
+		if (ex) {
+			/* Its okay to raise in llvmonly mode */
+			if (mono_llvm_only) {
+				mono_llvm_throw_exception ((MonoObject*)ex);
+			} else {
+				mono_set_pending_exception (ex);
+			}
+		}
 	}
 }
 
 /* Called from generated code to initialize a method */
 void
-mini_llvmonly_init_method (gpointer aot_module, guint32 method_index)
+mini_llvm_init_method (gpointer aot_module, guint32 method_index)
 {
 	MonoAotModule *amodule = (MonoAotModule *)aot_module;
 
-	init_llvmonly_method (amodule, method_index, NULL, NULL, FALSE);
+	init_llvmonly_method (amodule, method_index, NULL);
 }
 
 /* Same for gshared methods with a this pointer */
 void
-mini_llvmonly_init_gshared_method_this (gpointer aot_module, guint32 method_index, MonoObject *this_obj)
+mini_llvm_init_gshared_method_this (gpointer aot_module, guint32 method_index, MonoObject *this_obj)
 {
 	MonoAotModule *amodule = (MonoAotModule *)aot_module;
 	MonoClass *klass;
@@ -804,39 +808,38 @@ mini_llvmonly_init_gshared_method_this (gpointer aot_module, guint32 method_inde
 	g_assert (this_obj);
 	klass = this_obj->vtable->klass;
 
-	/*
-	 * We can't obtain the context from THIS_OBJ since it can point to a child class etc., so
-	 * we depend on these methods to be called indirectly which causes them to be added
-	 * to extra_methods.
-	 */
-	init_llvmonly_method (amodule, method_index, klass, NULL, TRUE);
+	init_llvmonly_method (amodule, method_index, klass);
 }
 
 /* Same for gshared methods with an mrgctx arg */
 void
-mini_llvmonly_init_gshared_method_mrgctx (gpointer aot_module, guint32 method_index, MonoMethodRuntimeGenericContext *rgctx)
+mini_llvm_init_gshared_method_mrgctx (gpointer aot_module, guint32 method_index, MonoMethodRuntimeGenericContext *rgctx)
 {
 	MonoAotModule *amodule = (MonoAotModule *)aot_module;
-	MonoGenericContext context = { NULL, NULL };
-	MonoClass *klass = rgctx->class_vtable->klass;
 
-	if (mono_class_is_ginst (klass))
-		context.class_inst = mono_class_get_generic_class (klass)->context.class_inst;
-	else if (mono_class_is_gtd (klass))
-		context.class_inst = mono_class_get_generic_container (klass)->context.class_inst;
-	context.method_inst = rgctx->method_inst;
-
-	init_llvmonly_method (amodule, method_index, rgctx->class_vtable->klass, &context, FALSE);
+	init_llvmonly_method (amodule, method_index, rgctx->class_vtable->klass);
 }
 
 /* Same for gshared methods with a vtable arg */
 void
-mini_llvmonly_init_gshared_method_vtable (gpointer aot_module, guint32 method_index, MonoVTable *vtable)
+mini_llvm_init_gshared_method_vtable (gpointer aot_module, guint32 method_index, MonoVTable *vtable)
 {
 	MonoAotModule *amodule = (MonoAotModule *)aot_module;
 	MonoClass *klass;
 
 	klass = vtable->klass;
 
-	init_llvmonly_method (amodule, method_index, klass, NULL, TRUE);
+	init_llvmonly_method (amodule, method_index, klass);
+}
+
+static GENERATE_GET_CLASS_WITH_CACHE (nullref, "System", "NullReferenceException")
+
+void
+mini_llvmonly_throw_nullref_exception (void)
+{
+	MonoClass *klass = mono_class_get_nullref_class ();
+
+	guint32 ex_token_index = m_class_get_type_token (klass) - MONO_TOKEN_TYPE_DEF;
+
+	mono_llvm_throw_corlib_exception (ex_token_index);
 }
