@@ -590,7 +590,7 @@ mono_thread_platform_create_thread (MonoThreadStart thread_fn, gpointer thread_d
 void mono_threads_platform_get_stack_bounds (guint8 **staddr, size_t *stsize);
 gboolean mono_threads_platform_is_main_thread (void);
 void mono_threads_platform_init (void);
-gboolean mono_threads_platform_in_critical_region (MonoNativeThreadId tid);
+gboolean mono_threads_platform_in_critical_region (THREAD_INFO_TYPE *info);
 gboolean mono_threads_platform_yield (void);
 void mono_threads_platform_exit (gsize exit_code);
 
@@ -811,6 +811,27 @@ mono_win32_leave_blocking_io_call (THREAD_INFO_TYPE *info, HANDLE io_handle);
 
 void
 mono_win32_interrupt_wait (PVOID thread_info, HANDLE native_thread_handle, DWORD tid);
+
+void
+mono_win32_abort_blocking_io_call (THREAD_INFO_TYPE *info);
+
+#define W32_DEFINE_LAST_ERROR_RESTORE_POINT \
+	DWORD _last_error_restore_point = GetLastError ();
+
+#define W32_UPDATE_LAST_ERROR_RESTORE_POINT \
+	_last_error_restore_point = GetLastError ();
+
+#define W32_RESTORE_LAST_ERROR_FROM_RESTORE_POINT \
+		/* Only restore if changed to prevent unecessary writes. */ \
+		if (GetLastError () != _last_error_restore_point) \
+			SetLastError (_last_error_restore_point);
+
+#else
+
+#define W32_DEFINE_LAST_ERROR_RESTORE_POINT /* nothing */
+#define W32_UPDATE_LAST_ERROR_RESTORE_POINT /* nothing */
+#define W32_RESTORE_LAST_ERROR_FROM_RESTORE_POINT /* nothing */
+
 #endif
 
 #endif /* __MONO_THREADS_H__ */
